# Sendmail.pm -- LocalMTA::Sendmail module file. See LocalMTA/Sendmail.pod.

# This file is part of LocalMTA-Sendmail, and is released under the GPLv2. Copyright 2013-2014 by Troye Johnson.

package LocalMTA::Sendmail;

use warnings;
use strict;

use Carp;

use File::Temp ();

our $VERSION = '0.02';

use Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(sendmail localmta_sendmail email_valid localmta_valid);

## used by sendmail() for template name (for File::Temp::tempfile())
my $LocalMTA_Sendmail = '/tmp/LocalMTASendmail.XXXX';

## used to verify emails
sub email_valid {
    return 1 if ((defined $_[0]) && ($_[0] =~ /^([a-zA-Z0-9_\.-]+)@([a-zA-Z0-9_\.-]+)$/)); # mail to Internet
    return 1 if ((defined $_[0]) && ($_[0] =~ /^([a-zA-Z0-9_\.-]+)$/)); # local
    return 0;
} # email_valid()

# ** localmta_valid: this is just another name for email_valid().
sub localmta_valid {
    my ($email_addr) = @_;

    # remove all characters safe for shells
    $email_addr =~ s/(@|-|_|\.)//g;

    return undef if ($email_addr =~ /\W/); # if bad chars remain, signal error
    return 1; # email_addr is good to go
} # localmta_valid


=item

    sendmail({mta_path => '/usr/sbin/exim4', local_mail_flg => '-bm'}, $to, $subject, $msg);

Returns: true or false

Parameters:
 - HASH, 'mta_path' must be present in parameter, it is the full path the MTA executable on your system. Do an 'ls -l /usr/sbin/sendmail' to see what it really is. 'local_mail_flag' must also be defined in the HASH. It is the flag used to tell the MTA to accept an email message over STDIN. Not all MTAs need that flag. Courier MTA does not need it, but Exim does (exim will refuse to send messages without '-bm' on command line).
 - $to, is the recipient. It the future, $to can may be an ARRAY reference, so that way you can send mail to multiple recipients at a time.
 - $subject, is the subject of the email message (string)
 - $msg, is the body of the email message to send via you local MTA

Note: you will need to configure the 'trusted users' list, so that the $full_from, and $from options will be honored. Else, the MTA will put whatever it thinks is reasonable. Only Exim4 needs this.

This module can send mail to multiple people at the same time, up to 128 recipients can be passed in.

=cut

sub sendmail {
    croak 'sendmail: must have six parameters' unless ($#_ == 5);
    croak 'sendmail: first param must be HASH reference' unless (ref $_[0]);

    my ($opts, $to, $full_from, $from, $subject, $msg) = (@_);

    # This prevents some "concatenation of undefined value" errors
    $opts->{local_mail_flag} = "" 
	unless defined $opts->{local_mail_flag};

    # email address must be valid, and not contain dangerous shell characters
    return 0 unless (localmta_valid($to));

    # MTA program must exist
    return 0 unless (-e $opts->{'mta_path'}); 

    # Build e-mail message headers, and append $msg
    my $finalmsg = 'To: ' . $to . "\n";
    $finalmsg .= 'Cc: ' . $opts->{'cc'} . "\n"
	if defined $opts->{'cc'};  # add xtra rcpnt if present -- xfmod
    $finalmsg .= 'From: ' .  $full_from .  ' <' . $from . ">\n";
    $finalmsg .= 'Subject: ' . $subject . "\n\n";
    $finalmsg .= $msg . "\n";  

    # Write mail $msg to $tmpfile
    local $SIG{'PIPE'} = 'IGNORE';
    local $SIG{'CHLD'} = 'IGNORE';
    
=item write tempfile code disabled to try out pipe approach

    my ($tmpfile, $tmpfilename)
	= File::Temp::tempfile($LocalMTA_Sendmail) or return 0;
    print $tmpfile $finalmsg or return 0;
    close $tmpfile; # flushes buffer

=cut

    # Get an output filehandle
    open( my $pipe_email, 
	  "| $opts->{'mta_path'} $opts->{'local_mail_flag'} $to")
	or return;

    print $pipe_email $finalmsg;
    close $pipe_email;

    my $return_value = $?;

    # Invoke MTA on message file we just wrote
    # `$opts->{'mta_path'} $opts->{'local_mail_flag'} -f $from $to < $tmpfilename >/dev/null 2>&1` ;


    # unlink $tmpfilename;

    # print "DEBUG: $opts->{'mta_path'} $opts->{'local_mail_flag'} -f $from $to < $tmpfilename >/dev/null 2>&1\n"; # debug  

    # Exim does not like to accept local messages. (See -bm and -oe[x] options in man page or Exim4 manual). Exim returns -1 (failure) even though message goes through. We account for that here.
    my $good_retval = -1;
    if ( $return_value >= $good_retval || $return_value >= 0)  {
	return 1; # success
    } 

    return 0; # failure
} # sendmail


# ** localmta_sendmail: just another name for LocalMTA::Sendmail::sendmail(). See docs for sendmail(). This is same function, but with a different name.

sub localmta_sendmail {
    return sendmail(@_);
}


1; # ok load
